<?php
/* vim: set expandtab tabstop=4 shiftwidth=4: */
// +----------------------------------------------------------------------+
// | PHP version 4.0                                                      |
// +----------------------------------------------------------------------+
// | Copyright (c) 1997-2003 The PHP Group                                |
// +----------------------------------------------------------------------+
// | This source file is subject to version 2.0 of the PHP license,       |
// | that is bundled with this package in the file LICENSE, and is        |
// | available at through the world-wide-web at                           |
// | http://www.php.net/license/2_02.txt.                                 |
// | If you did not receive a copy of the PHP license and are unable to   |
// | obtain it through the world-wide-web, please send a note to          |
// | license@php.net so we can mail you a copy immediately.               |
// +----------------------------------------------------------------------+
// | Authors: Ikkei SHIMOMURA                              |
// +----------------------------------------------------------------------+
// $Id: http.class.php,v 1.10 2003/01/10 06:51:07 ikkei Exp $



/**
* HTTP protocol class
*
* @author        $Author$
* @version        1.1
* @since        PHP 4.3.0
*/
class HTTP
{
    // {{{ propaties

    /**
    * file pointer of socket
    * @since    1.1
    * @access    private
    */
    var $fp = NULL;

    // }}}
    
    // {{{ constructor
    /** Constructor
     * @since        1.1
     * @access        public
     */
    function HTTP()
    {
        // do nothing yet.
    }
    // }}}
    
    // {{{ send()
    /**
     * sending data
     * @param        object HTTP_Request        $req    HTTP_Request
     * @since        1.1
     * @access        public
     */
    function send($req)
    {
        $this->close();

        $this->fp = @fsockopen($req->host, $req->port);

        if (! $this->fp) {
            return false;
        }

        fwrite($this->fp, $req->toString());
        flush($this->fp);
    }
    // }}}
    
    // {{{ recv()
    /**
     * recieving data
     * @since        1.1
     * @access        public
     * @return        object HTTP_Response
     */
    function recv()
    {
        $res = new HTTP_Response();
        $res->parse($this->fp);

        return $res;
    }
    // }}}

    // {{{ close()
    /**
     * close socket if it exists
     * @since        1.1
     * @access        private
     */
    function close()
    {
        if ($this->fp) {
            fclose($this->fp);
            $this->fp = NULL;
        }
    }
    // }}}
    
    // {{{ get()
    /**
     * request data by 'GET' method
     * @param        object HTTP_Request        $req    HTTP_Request class
     * @since        1.1
     * @access        public
     * @return        object HTTP_Response
     */
    function get($req)
    {
        $req->setMethod("GET");
        $this->send($req);
        
        return $this->recv();
    }
    // }}}

    // {{{ head()
    /**
     * request data by 'HEAD' method
     * @param        object HTTP_Request        $req    HTTP_Request class
     * @since        1.1
     * @access        public
     * @return        object HTTP_Response
     */
    function head($req)
    {
        $req->setMethod("HEAD");
        $this->send($req);
        
        return $this->recv();
    }
    // }}}
    
    // {{{ post()
    /**
     * request data by'POST' method
     * @param        object HTTP_Request        $req    HTTP_Request class
     * @since        1.1
     * @access        public
     * @return        object HTTP_Response
     */
    function post($req)
    {
        $req->setMethod("POST");
        $this->send($req);

        return $this->recv();
    }
    // }}}
}
// }}}
?>


