<?php
/* vim: set expandtab tabstop=4 shiftwidth=4: */
// +----------------------------------------------------------------------+
// | PHP version 4.0                                                      |
// +----------------------------------------------------------------------+
// | Copyright (c) 1997-2003 The PHP Group                                |
// +----------------------------------------------------------------------+
// | This source file is subject to version 2.0 of the PHP license,       |
// | that is bundled with this package in the file LICENSE, and is        |
// | available at through the world-wide-web at                           |
// | http://www.php.net/license/2_02.txt.                                 |
// | If you did not receive a copy of the PHP license and are unable to   |
// | obtain it through the world-wide-web, please send a note to          |
// | license@php.net so we can mail you a copy immediately.               |
// +----------------------------------------------------------------------+
// | Authors: Ikkei SHIMOMURA <tea@kcn.ne.jp>                             |
// +----------------------------------------------------------------------+
// $Id$



/**
* Create, HTTP Request Message 
*
* @author        $Author$
* @version        1.1
* @since        PHP 4.3.0
*/
class HTTP_Request
{
    // {{{ propaties

    /**
    * Array containing the http request header.
    * @since    1.1
    * @var        array
    * @access    private
    */
    var $headers = array();
    

    /**
    * Array containing paramaetrs of GET or POST request.
    * @since    1.1
    * @var        array
    * @access    private
    */
    var $param   = array();

    /**
    * Hostname where is there server you connect.
    * this value is used by the value of Host: header field.
    * @since    1.1
    * @var        string
    * @access    private
    */
    var $host    = "";

    /**
    * Hostname where is there server you connect.
    * this value is used by the value of Host: header field.
    * @since    1.1
    * @var        string
    * @access    private
    */
    var $path    = "/";

    /**
    * port number of target host. by default the http port number is 80
    * @since    1.1
    * @var        integer
    * @access    private
    */
    var $port    = 80;
    
    
    /* Not Implement Yet */
    var $user    = "";
    var $pass    = "";
    
    /**
    * Request Method String
    * @since    1.1
    * @var        string
    * @access    private
    */
    var $method  = "GET";
    

    /**
    * Version Number of HTTP
    * @since    1.1
    * @var        string
    * @access    private
    */
    var $version = "1.1";
    
    // }}}

    // {{{ constructor
    /** Class constructor
     * @param        string        $url        URL String
     * @param        string        $method        (optional)Request Method default to 'GET'
     * @access        public
     */
    function HTTP_Request($url, $method = "GET")
    {
        $tmp = parse_url($url);

        if (!ereg("^http$", $tmp['scheme'])) {
            return false;
        }

        $this->host   = $tmp['host'];
        $this->path   = (!empty($tmp['path']) ) ? $tmp['path']  : "";
        // $this->user   = (!empty($tmp['user']) ) ? $tmp['user']  : "";
        // $this->pass   = (!empty($tmp['pass']) ) ? $tmp['pass']  : "";
        $this->query  = (!empty($tmp['query'])) ? $tmp['query'] : "";
        if (!empty($tmp['port'])) {
            $this->port = $tmp['port'];
        }

        $this->setMethod($method);
    }
    // }}}

    // {{{ getMethod()
    /**
     * Returns the current setting Request Method string
     *
     * @since        1.0
     * @access        public
     * @return        string
     */
    function getMethod() // [TODO] rename to getRequestMethod
    {
        return strtoupper($this->method);
    }
    // }}}

    // {{{ setMethod()
    /**
     * Setting Request Method
     *
     * @param        string        $method        RequestMethod string
     * @since        1.0
     * @access        public
     */
    function setMethod($method) // [TODO] rename to setRequestMethod
    {
        $this->method = strtoupper($method);
    }
    // }}}

    // {{{ getheader()
    /** Returns the value of header by given key name
     *
     * @param        string        $k        Key name of header
     * @since        1.0
     * @access        public
     * @return        string
     * @throw
     */
    function getHeader($k)
    {
        return $this->headers[ucfirst(strtolower($k))];
    }
    // }}}

    // {{{ setHeader()
    /** Setting value into Headers
     *
     * @param        string        $k        Key name of header
     * @param        string        $v        the value
     * @since        1.0
     * @access        public
     */
    function setHeader($k, $v)
    {
        $this->headers[ucfirst(strtolower($k))] = $v;
    }
    // }}}

    // {{{ getParam()
    /** Returns the value of param by given key name
     *
     * @param        string        $k        Key name of param
     * @since        1.0
     * @access        public
     * @return        string
     * @throw
     */
    function getParam($k)
    {
        return $this->param[$k];
    }

    // {{{ setParam()
    /** Setting value into Param
     *
     * @param        string        $k        Key name of param
     * @param        string        $v        the value
     * @since        1.0
     * @access        public
     */
    function setParam($k, $v)
    {
        $this->param[$k] = $v;
    }
    // }}}

    // {{{ getDate()
    /**
     * Returns the formated request param
     * @since        1.0
     * @access        public
     * @return        string
     * @throw
     */
    function getData() // [TODO] should refactoring the func name ?
    {
        $data = "";
        foreach ($this->param as $k => $v) {
            $data .= $k . "=" . urlencode($v) . ";";
        }
        return $data;
    }
    // }}}

    // {{{ toString()
    /**
     * Return the request message string
     * 
     * @since        1.0
     * @access        public
     * @return        string
     * @throw
     */
    function toString()
    {
        $str   = "";
        $query = "";
        $data  = "";

        if (empty($this->headers['Host'])) {
            $this->setHeader('Host', $this->host);
        }

        if (empty($this->headers['Connection'])) {
            $this->setHeader('Connection', 'close');
        }

        switch ($this->getMethod()) {
        case "GET":
            if ($this->query) {
            $query = "?" . $this->query;
            } else {
            $query = "?" . $this->getData();
            }
            break;
        case "POST":
            $data = $this->getData();
            $this->setHeader('Content-type', 'x-www-form-urlencoded');
            $this->setHeader('Content-length', strlen($data));
            break;
        }


        // Generate HTTP Request
        $str = "$this->method $this->path$query HTTP/$this->version\r\n";

        foreach ($this->headers as $k => $v) {
            $str .= "$k: $v\r\n";
        }

        $str .= "\r\n";

        if ($this->method == "POST") {
            $str .= $data;
        }

        return $str;
    }
    // }}}
}
// }}}
?>

