<?php
/* vim: set expandtab tabstop=4 shiftwidth=4: */
// +----------------------------------------------------------------------+
// | PHP version 4.0                                                      |
// +----------------------------------------------------------------------+
// | Copyright (c) 1997-2003 The PHP Group                                |
// +----------------------------------------------------------------------+
// | This source file is subject to version 2.0 of the PHP license,       |
// | that is bundled with this package in the file LICENSE, and is        |
// | available at through the world-wide-web at                           |
// | http://www.php.net/license/2_02.txt.                                 |
// | If you did not receive a copy of the PHP license and are unable to   |
// | obtain it through the world-wide-web, please send a note to          |
// | license@php.net so we can mail you a copy immediately.               |
// +----------------------------------------------------------------------+
// | Authors: Ikkei SHIMOMURA <tea@kcn.ne.jp>                             |
// +----------------------------------------------------------------------+
// $Id$

/**
* Response class of HTTP message
* (this can read only single message, can not parse MIME message)
*
* @author        $Author$
* @version        1.1
* @since        PHP 4.3.0
*/
class HTTP_Response{
    /**
    * status code
    * @since    1.1
    * @var        string
    * @access    private
    */
    var $status  = "";

    /**
    * Array of headers
    * @since    1.1
    * @var        array
    * @access    private
    */
    var $headers = array();
    

    /**
    * the resource of requested
    * @since    1.1
    * @var        string
    * @access    private
    */
    var $body    = "";
    
    /** Class constructor
     * @param        string    $fp        socket
     * @since        1.1
     * @access        public
     */
    function HTTP_Reqponse($fp=NULL)
    {
        if ($fp) {
            $this->parse($fp);
        }
    }
    
    /**
     * Parse data and set it on param
     * @param        resource    $fp        integer
     * @since        1.1
     * @access        public
     */
    function parse($fp)
    {
        if (!$fp) {
            return false;
        }

        // parse status line
        $this->status = fgets($fp, 1024);

        // parse headers lines
        $tmp = "";
        do {
            $tmp = fgets($fp, 1024);
            $tmp = rtrim($tmp);
          
            if (ereg("^([^\:]+)\:(.+)$", $tmp, $ret)) {
                $this->setHeader($ret[1], $ret[2]);
            }
        } while ($tmp);

        // parse body
        while (!feof($fp)) {
            $this->body .= fgets($fp, 1024);
        }
    }

    /* [NOTE] ... about parse method.
     *
     * $fp is given by HTTP class.
     * Q. Where should I close this ?
     *    Are there any defien pattern to solve this ?
     *
     * Ans1. HTTP parse data and set it on HTTP_Request.
     */
    
    /**
     * Returns Status string
     * @since        1.1
     * @access        public
     * @return        string
     */
    function getStatus()
    {
        return $this->status;
    }
    
    /**
     * Setting param on header
     * @param        string        $k        key name of header
     * @param        string        $v        the value
     * @since        1.1
     * @access        public
     */
    function setHeader($k, $v)
    {
        $this->headers[ucfirst(strtolower($k))] = $v;
    }
    
    
    /**
     * Returns value of header by given key name
     * @param        string        $k        key name of header
     * @since        1.1
     * @access        public
     * @return        string
     * @throw
     */
    function getHeader($k)
    {
        return $this->headers[ucfirst(strtolower($k))];
    }
    
    
    /**
     * Return resource of requested
     * @since        1.1
     * @access        public
     * @return        string
     */
    function getBody()
    {
        return $this->body;
    }
    
    
    /**
     * Return Modified time of resource on url
     * @since        1.1
     * @access        public
     * @return        string
     * @throw
     */
    function mtime()
    {
        return strtotime($this->getHeader('Data'));
    }
}

?>
